@@ -65,13 +65,20 @@ struct guac_client_info {
      * NULL-terminated array of client-supported audio mimetypes. If the client
      * does not support audio at all, this will be NULL.
      */
-    const char** audio_mimetypes;
+    char** audio_mimetypes;
 
     /**
      * NULL-terminated array of client-supported video mimetypes. If the client
      * does not support video at all, this will be NULL.
      */
-    const char** video_mimetypes;
+    char** video_mimetypes;
+
+    /**
+     * NULL-terminated array of client-supported image mimetypes. Though all
+     * supported image mimetypes will be listed here, it can be safely assumed
+     * that all clients will support at least "image/png" and "image/jpeg".
+     */ 
+    char** image_mimetypes;
 
     /**
      * The DPI of the physical remote display if configured for the optimal
@@ -479,11 +486,18 @@ void guac_client_free(guac_client* client);
  * initial handler lookup table defined in client-handlers.c. The intial
  * handlers will in turn call the client's handler (if defined).
  *
- * @param client The proxy client whose handlers should be called.
- * @param instruction The instruction to pass to the proxy client via the
- *                    appropriate handler.
+ * @param client
+ *     The proxy client whose handlers should be called.
+ *
+ * @param instruction
+ *     The instruction to pass to the proxy client via the appropriate handler.
+ *
+ * @return
+ *     Non-negative if the instruction was handled successfully, or negative
+ *     if an error occurred.
  */
-int guac_client_handle_instruction(guac_client* client, guac_instruction* instruction);
+int guac_client_handle_instruction(guac_client* client,
+        guac_instruction* instruction);
 
 /**
  * Writes a message in the log used by the given client. The logger used will
@@ -689,11 +703,73 @@ void guac_client_stream_png(guac_client* client, guac_socket* socket,
  *
  * @param surface
  *     A Cairo surface containing the image data to be streamed.
+ *
+ * @param quality
+ *     The JPEG image quality, which must be an integer value between 0 and
+ *     100 inclusive.
  */
 void guac_client_stream_jpeg(guac_client* client, guac_socket* socket,
         guac_composite_mode mode, const guac_layer* layer, int x, int y,
         cairo_surface_t* surface, int quality);
 
+/**
+ * Streams the image data of the given surface over an image stream ("img"
+ * instruction) as WebP-encoded data at the given quality. The image stream
+ * will be automatically allocated and freed. If the server does not support
+ * WebP, this function has no effect, so be sure to check the result of
+ * guac_client_supports_webp() prior to calling this function.
+ *
+ * @param client
+ *     The Guacamole client from which the image stream should be allocated.
+ *
+ * @param socket
+ *     The socket over which instructions associated with the image stream
+ *     should be sent.
+ *
+ * @param mode
+ *     The composite mode to use when rendering the image over the given layer.
+ *
+ * @param layer
+ *     The destination layer.
+ *
+ * @param x
+ *     The X coordinate of the upper-left corner of the destination rectangle
+ *     within the given layer.
+ *
+ * @param y
+ *     The Y coordinate of the upper-left corner of the destination rectangle
+ *     within the given layer.
+ *
+ * @param surface
+ *     A Cairo surface containing the image data to be streamed.
+ *
+ * @param quality
+ *     The WebP image quality, which must be an integer value between 0 and 100
+ *     inclusive. For lossy images, larger values indicate improving quality at
+ *     the expense of larger file size. For lossless images, this dictates the
+ *     quality of compression, with larger values producing smaller files at
+ *     the expense of speed.
+ *
+ * @param lossless
+ *     Zero to encode a lossy image, non-zero to encode losslessly.
+ */
+void guac_client_stream_webp(guac_client* client, guac_socket* socket,
+        guac_composite_mode mode, const guac_layer* layer, int x, int y,
+        cairo_surface_t* surface, int quality, int lossless);
+
+/**
+ * Returns whether the given client supports WebP. If the client does not
+ * support WebP, or the server cannot encode WebP images, zero is returned.
+ *
+ * @param client
+ *     The Guacamole client to check for WebP support.
+ *
+ * @return
+ *     Non-zero if the given client claims to support WebP and the server has
+ *     been built with WebP support, zero otherwise.
+ */
+int guac_client_supports_webp(guac_client* client);
+
 /**
  * The default Guacamole client layer, layer 0.
  */
