@@ -130,10 +130,19 @@ int guac_protocol_send_log(guac_socket* socket, const char* format, ...);
  * If an error occurs sending the instruction, a non-zero value is
  * returned, and guac_error is set appropriately.
  *
- * @param socket The guac_socket connection to use.
- * @param format A printf-style format string to log.
- * @param ap The va_list containing the arguments to be used when filling the
- *           format string for printing.
+ * @param socket
+ *     The guac_socket connection to use.
+ *
+ * @param format
+ *     A printf-style format string to log.
+ *
+ * @param args
+ *     The va_list containing the arguments to be used when filling the
+ *     format string for printing.
+ *
+ * @return
+ *     Zero if the instruction was sent successfully, non-zero if an error
+ *     occurs.
  */
 int vguac_protocol_send_log(guac_socket* socket, const char* format,
         va_list args);
@@ -295,15 +304,20 @@ int guac_protocol_send_undefine(guac_socket* socket,
  * If an error occurs sending the instruction, a non-zero value is
  * returned, and guac_error is set appropriately.
  *
- * @param socket The guac_socket connection to use.
- * @param stream The stream to use.
- * @param channel The index of the audio channel to use.
- * @param mimetype The mimetype of the data being sent.
- * @param duration The duration of the sound being sent, in milliseconds.
- * @return Zero on success, non-zero on error.
+ * @param socket
+ *     The guac_socket connection to use when sending the audio instruction.
+ *
+ * @param stream
+ *     The stream to use for future audio data.
+ *
+ * @param mimetype
+ *     The mimetype of the audio data which will be sent over the given stream.
+ *
+ * @return
+ *     Zero on success, non-zero on error.
  */
 int guac_protocol_send_audio(guac_socket* socket, const guac_stream* stream,
-        int channel, const char* mimetype, double duration);
+        const char* mimetype);
 
 /**
  * Sends a file instruction over the given guac_socket connection.
@@ -350,7 +364,7 @@ int guac_protocol_send_pipe(guac_socket* socket, const guac_stream* stream,
  * @return Zero on success, non-zero on error.
  */
 int guac_protocol_send_blob(guac_socket* socket, const guac_stream* stream,
-        void* data, int count);
+        const void* data, int count);
 
 /**
  * Sends an end instruction over the given guac_socket connection.
@@ -370,15 +384,23 @@ int guac_protocol_send_end(guac_socket* socket, const guac_stream* stream);
  * If an error occurs sending the instruction, a non-zero value is
  * returned, and guac_error is set appropriately.
  *
- * @param socket The guac_socket connection to use.
- * @param stream The stream to use.
- * @param layer The destination layer.
- * @param mimetype The mimetype of the data being sent.
- * @param duration The duration of the video being sent, in milliseconds.
- * @return Zero on success, non-zero on error.
+ * @param socket
+ *     The guac_socket connection to use when sending the video instruction.
+ *
+ * @param stream
+ *     The stream to use for future video data.
+ *
+ * @param layer
+ *     The destination layer on which the streamed video should be played.
+ *
+ * @param mimetype
+ *     The mimetype of the video data which will be sent over the given stream.
+ *
+ * @return
+ *     Zero on success, non-zero on error.
  */
 int guac_protocol_send_video(guac_socket* socket, const guac_stream* stream,
-        const guac_layer* layer, const char* mimetype, double duration);
+        const guac_layer* layer, const char* mimetype);
 
 /* DRAWING INSTRUCTIONS */
 
@@ -613,6 +635,9 @@ int guac_protocol_send_lstroke(guac_socket* socket,
  * @param layer
  *     The destination layer.
  *
+ * @param mimetype
+ *     The mimetype of the image data being sent.
+ *
  * @param x
  *     The X coordinate of the upper-left corner of the destination rectangle
  *     within the destination layer, in pixels.
