@@ -153,7 +153,7 @@ int guac_protocol_send_arc(guac_socket* socket, const guac_layer* layer,
 }
 
 int guac_protocol_send_audio(guac_socket* socket, const guac_stream* stream,
-        int channel, const char* mimetype, double duration) {
+        const char* mimetype) {
 
     int ret_val;
 
@@ -162,11 +162,7 @@ int guac_protocol_send_audio(guac_socket* socket, const guac_stream* stream,
            guac_socket_write_string(socket, "5.audio,")
         || __guac_socket_write_length_int(socket, stream->index)
         || guac_socket_write_string(socket, ",")
-        || __guac_socket_write_length_int(socket, channel)
-        || guac_socket_write_string(socket, ",")
         || __guac_socket_write_length_string(socket, mimetype)
-        || guac_socket_write_string(socket, ",")
-        || __guac_socket_write_length_double(socket, duration)
         || guac_socket_write_string(socket, ";");
     guac_socket_instruction_end(socket);
 
@@ -175,7 +171,7 @@ int guac_protocol_send_audio(guac_socket* socket, const guac_stream* stream,
 }
 
 int guac_protocol_send_blob(guac_socket* socket, const guac_stream* stream,
-        void* data, int count) {
+        const void* data, int count) {
 
     int base64_length = (count + 2) / 3 * 4;
 
@@ -1078,7 +1074,7 @@ int guac_protocol_send_undefine(guac_socket* socket,
 }
 
 int guac_protocol_send_video(guac_socket* socket, const guac_stream* stream,
-        const guac_layer* layer, const char* mimetype, double duration) {
+        const guac_layer* layer, const char* mimetype) {
 
     int ret_val;
 
@@ -1090,8 +1086,6 @@ int guac_protocol_send_video(guac_socket* socket, const guac_stream* stream,
         || __guac_socket_write_length_int(socket, layer->index)
         || guac_socket_write_string(socket, ",")
         || __guac_socket_write_length_string(socket, mimetype)
-        || guac_socket_write_string(socket, ",")
-        || __guac_socket_write_length_double(socket, duration)
         || guac_socket_write_string(socket, ";");
     guac_socket_instruction_end(socket);
 
