@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Copyright (C) 2015 Glyptodon LLC
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -129,6 +129,8 @@ const char* GUAC_CLIENT_ARGS[] = {
     "enable-full-window-drag",
     "enable-desktop-composition",
     "enable-menu-animations",
+    "preconnection-id",
+    "preconnection-blob",
 
 #ifdef ENABLE_COMMON_SSH
     "enable-sftp",
@@ -178,6 +180,8 @@ enum RDP_ARGS_IDX {
     IDX_ENABLE_FULL_WINDOW_DRAG,
     IDX_ENABLE_DESKTOP_COMPOSITION,
     IDX_ENABLE_MENU_ANIMATIONS,
+    IDX_PRECONNECTION_ID,
+    IDX_PRECONNECTION_BLOB,
 
 #ifdef ENABLE_COMMON_SSH
     IDX_ENABLE_SFTP,
@@ -286,19 +290,13 @@ BOOL rdp_freerdp_pre_connect(freerdp* instance) {
     /* If audio enabled, choose an encoder */
     if (guac_client_data->settings.audio_enabled) {
 
-        guac_client_data->audio = guac_audio_stream_alloc(client, NULL);
+        guac_client_data->audio = guac_audio_stream_alloc(client, NULL,
+                GUAC_RDP_AUDIO_RATE,
+                GUAC_RDP_AUDIO_CHANNELS,
+                GUAC_RDP_AUDIO_BPS);
 
-        /* If an encoding is available, load the sound plugin */
-        if (guac_client_data->audio != NULL) {
-
-            /* Load sound plugin */
-            if (freerdp_channels_load_plugin(channels, instance->settings,
-                        "guacsnd", guac_client_data->audio))
-                guac_client_log(client, GUAC_LOG_WARNING,
-                        "Failed to load guacsnd plugin. Audio will not work.");
-
-        }
-        else
+        /* Warn if no audio encoding is available */
+        if (guac_client_data->audio == NULL)
             guac_client_log(client, GUAC_LOG_INFO,
                     "No available audio encoding. Sound disabled.");
 
@@ -318,7 +316,7 @@ BOOL rdp_freerdp_pre_connect(freerdp* instance) {
 
     }
 
-    /* If RDPDR required, load it */
+    /* If RDPSND/RDPDR required, load them */
     if (guac_client_data->settings.printing_enabled
         || guac_client_data->settings.drive_enabled
         || guac_client_data->settings.audio_enabled) {
@@ -327,7 +325,16 @@ BOOL rdp_freerdp_pre_connect(freerdp* instance) {
         if (freerdp_channels_load_plugin(channels, instance->settings,
                     "guacdr", client))
             guac_client_log(client, GUAC_LOG_WARNING,
-                    "Failed to load guacdr plugin. Drive redirection and printing will not work.");
+                    "Failed to load guacdr plugin. Drive redirection and "
+                    "printing will not work. Sound MAY not work.");
+
+        /* Load RDPSND plugin */
+        if (freerdp_channels_load_plugin(channels, instance->settings,
+                    "guacsnd", client))
+            guac_client_log(client, GUAC_LOG_WARNING,
+                    "Failed to load guacsnd alongside guacdr plugin. Sound "
+                    "will not work. Drive redirection and printing MAY not "
+                    "work.");
 
     }
 
@@ -770,6 +777,45 @@ int guac_client_init(guac_client* client, int argc, char** argv) {
                 argv[IDX_WIDTH], settings->color_depth);
     }
 
+    /* Preconnection ID */
+    settings->preconnection_id = -1;
+    if (argv[IDX_PRECONNECTION_ID][0] != '\0') {
+
+        /* Parse preconnection ID, warn if invalid */
+        int preconnection_id = atoi(argv[IDX_PRECONNECTION_ID]);
+        if (preconnection_id < 0)
+            guac_client_log(client, GUAC_LOG_WARNING,
+                    "Ignoring invalid preconnection ID: %i",
+                    preconnection_id);
+
+        /* Otherwise, assign specified ID */
+        else {
+            settings->preconnection_id = preconnection_id;
+            guac_client_log(client, GUAC_LOG_DEBUG,
+                    "Preconnection ID: %i", settings->preconnection_id);
+        }
+
+    }
+
+    /* Preconnection BLOB */
+    settings->preconnection_blob = NULL;
+    if (argv[IDX_PRECONNECTION_BLOB][0] != '\0') {
+        settings->preconnection_blob = strdup(argv[IDX_PRECONNECTION_BLOB]);
+        guac_client_log(client, GUAC_LOG_DEBUG,
+                "Preconnection BLOB: \"%s\"", settings->preconnection_blob);
+    }
+
+#ifndef HAVE_RDPSETTINGS_SENDPRECONNECTIONPDU
+    /* Warn if support for the preconnection BLOB / ID is absent */
+    if (settings->preconnection_blob != NULL
+            || settings->preconnection_id != -1) {
+        guac_client_log(client, GUAC_LOG_WARNING,
+                "Installed version of FreeRDP lacks support for the "
+                "preconnection PDU. The specified preconnection BLOB and/or "
+                "ID will be ignored.");
+    }
+#endif
+
     /* Audio enable/disable */
     guac_client_data->settings.audio_enabled =
         (strcmp(argv[IDX_DISABLE_AUDIO], "true") != 0);
