@@ -87,6 +87,27 @@
  */
 #define GUAC_RDP_CLIPBOARD_MAX_LENGTH 262144
 
+/**
+ * Initial rate of audio to stream, in Hz. If the RDP server uses a different
+ * value, the Guacamole audio stream will simply be reset appropriately.
+ */
+#define GUAC_RDP_AUDIO_RATE 44100
+
+/**
+ * The number of channels to stream for audio. If the RDP server uses a
+ * different value, the Guacamole audio stream will simply be reset
+ * appropriately.
+ */
+#define GUAC_RDP_AUDIO_CHANNELS 2
+
+/**
+ * The number of bits per sample within the audio stream. If the RDP server
+ * uses a different value, the Guacamole audio stream will simply be reset
+ * appropriately.
+ */
+#define GUAC_RDP_AUDIO_BPS 16
+
+
 /**
  * Client data that will remain accessible through the guac_client.
  * This should generally include data commonly used by Guacamole handlers.
