@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Glyptodon LLC
+ * Copyright (C) 2015 Glyptodon LLC
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -44,8 +44,7 @@
 /* MESSAGE HANDLERS */
 
 void guac_rdpsnd_formats_handler(guac_rdpsndPlugin* rdpsnd,
-        guac_audio_stream* audio, wStream* input_stream, 
-        guac_rdpsnd_pdu_header* header) {
+        wStream* input_stream, guac_rdpsnd_pdu_header* header) {
 
     int server_format_count;
     int server_version;
@@ -55,8 +54,12 @@ void guac_rdpsnd_formats_handler(guac_rdpsndPlugin* rdpsnd,
     int output_body_size;
     unsigned char* output_stream_end;
 
-    rdp_guac_client_data* guac_client_data =
-        (rdp_guac_client_data*) audio->client->data;
+    /* Get associated client data */
+    guac_client* client = rdpsnd->client;
+    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
+
+    /* Get audio stream from client data */
+    guac_audio_stream* audio = client_data->audio;
 
     /* Format header */
     Stream_Seek(input_stream, 14);
@@ -89,72 +92,86 @@ void guac_rdpsnd_formats_handler(guac_rdpsndPlugin* rdpsnd,
     Stream_Write_UINT16(output_stream, 6);
     Stream_Write_UINT8(output_stream,  0);
 
-    /* Check each server format, respond if supported */
-    for (i=0; i < server_format_count; i++) {
+    /* Check each server format, respond if supported and audio is enabled */
+    if (audio != NULL) {
+        for (i=0; i < server_format_count; i++) {
 
-        unsigned char* format_start;
+            unsigned char* format_start;
 
-        int format_tag;
-        int channels;
-        int rate;
-        int bps;
-        int body_size;
+            int format_tag;
+            int channels;
+            int rate;
+            int bps;
+            int body_size;
 
-        /* Remember position in stream */
-        Stream_GetPointer(input_stream, format_start);
+            /* Remember position in stream */
+            Stream_GetPointer(input_stream, format_start);
 
-        /* Read format */
-        Stream_Read_UINT16(input_stream, format_tag);
-        Stream_Read_UINT16(input_stream, channels);
-        Stream_Read_UINT32(input_stream, rate);
-        Stream_Seek_UINT32(input_stream);
-        Stream_Seek_UINT16(input_stream);
-        Stream_Read_UINT16(input_stream, bps);
+            /* Read format */
+            Stream_Read_UINT16(input_stream, format_tag);
+            Stream_Read_UINT16(input_stream, channels);
+            Stream_Read_UINT32(input_stream, rate);
+            Stream_Seek_UINT32(input_stream);
+            Stream_Seek_UINT16(input_stream);
+            Stream_Read_UINT16(input_stream, bps);
 
-        /* Skip past extra data */
-        Stream_Read_UINT16(input_stream, body_size);
-        Stream_Seek(input_stream, body_size);
+            /* Skip past extra data */
+            Stream_Read_UINT16(input_stream, body_size);
+            Stream_Seek(input_stream, body_size);
 
-        /* If PCM, accept */
-        if (format_tag == WAVE_FORMAT_PCM) {
+            /* If PCM, accept */
+            if (format_tag == WAVE_FORMAT_PCM) {
 
-            /* If can fit another format, accept it */
-            if (rdpsnd->format_count < GUAC_RDP_MAX_FORMATS) {
+                /* If can fit another format, accept it */
+                if (rdpsnd->format_count < GUAC_RDP_MAX_FORMATS) {
 
-                /* Add channel */
-                int current = rdpsnd->format_count++;
-                rdpsnd->formats[current].rate     = rate;
-                rdpsnd->formats[current].channels = channels;
-                rdpsnd->formats[current].bps      = bps;
+                    /* Add channel */
+                    int current = rdpsnd->format_count++;
+                    rdpsnd->formats[current].rate     = rate;
+                    rdpsnd->formats[current].channels = channels;
+                    rdpsnd->formats[current].bps      = bps;
 
-                /* Log format */
-                guac_client_log(audio->client, GUAC_LOG_INFO,
-                        "Accepted format: %i-bit PCM with %i channels at "
-                        "%i Hz",
-                        bps, channels, rate);
+                    /* Log format */
+                    guac_client_log(client, GUAC_LOG_INFO,
+                            "Accepted format: %i-bit PCM with %i channels at "
+                            "%i Hz",
+                            bps, channels, rate);
 
-                /* Queue format for sending as accepted */
-                Stream_EnsureRemainingCapacity(output_stream, 18 + body_size);
-                Stream_Write(output_stream, format_start, 18 + body_size);
+                    /* Ensure audio stream is configured to use accepted
+                     * format */
+                    guac_audio_stream_reset(audio, NULL, rate, channels, bps);
 
-                /* 
-                 * BEWARE that using Stream_EnsureRemainingCapacity means that any
-                 * pointers returned via Stream_GetPointer on output_stream are invalid.
-                 */
+                    /* Queue format for sending as accepted */
+                    Stream_EnsureRemainingCapacity(output_stream,
+                            18 + body_size);
+                    Stream_Write(output_stream, format_start, 18 + body_size);
+
+                    /*
+                     * BEWARE that using Stream_EnsureRemainingCapacity means
+                     * that any pointers returned via Stream_GetPointer on
+                     * output_stream are invalid.
+                     */
+
+                }
+
+                /* Otherwise, log that we dropped one */
+                else
+                    guac_client_log(client, GUAC_LOG_INFO,
+                            "Dropped valid format: %i-bit PCM with %i "
+                            "channels at %i Hz",
+                            bps, channels, rate);
 
             }
 
-            /* Otherwise, log that we dropped one */
-            else
-                guac_client_log(audio->client, GUAC_LOG_INFO,
-                        "Dropped valid format: %i-bit PCM with %i channels at "
-                        "%i Hz",
-                        bps, channels, rate);
-
         }
-
     }
 
+    /* Otherwise, ignore all supported formats as we do not intend to actually
+     * receive audio */
+    else
+        guac_client_log(client, GUAC_LOG_DEBUG,
+                "Audio explicitly disabled. Ignoring supported formats.");
+
     /* Calculate size of PDU */
     output_body_size = Stream_GetPosition(output_stream) - 4;
     Stream_GetPointer(output_stream, output_stream_end);
@@ -171,7 +188,7 @@ void guac_rdpsnd_formats_handler(guac_rdpsndPlugin* rdpsnd,
     Stream_SetPointer(output_stream, output_stream_end);
 
     /* Send accepted formats */
-    pthread_mutex_lock(&(guac_client_data->rdp_lock));
+    pthread_mutex_lock(&(client_data->rdp_lock));
     svc_plugin_send((rdpSvcPlugin*)rdpsnd, output_stream);
 
     /* If version greater than 6, must send Quality Mode PDU */
@@ -188,20 +205,20 @@ void guac_rdpsnd_formats_handler(guac_rdpsndPlugin* rdpsnd,
         svc_plugin_send((rdpSvcPlugin*)rdpsnd, output_stream);
     }
 
-    pthread_mutex_unlock(&(guac_client_data->rdp_lock));
+    pthread_mutex_unlock(&(client_data->rdp_lock));
 
 }
 
 /* server is getting a feel of the round trip time */
 void guac_rdpsnd_training_handler(guac_rdpsndPlugin* rdpsnd,
-        guac_audio_stream* audio, wStream* input_stream,
-        guac_rdpsnd_pdu_header* header) {
+        wStream* input_stream, guac_rdpsnd_pdu_header* header) {
 
     int data_size;
     wStream* output_stream;
 
-    rdp_guac_client_data* guac_client_data =
-        (rdp_guac_client_data*) audio->client->data;
+    /* Get associated client data */
+    guac_client* client = rdpsnd->client;
+    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
 
     /* Read timestamp and data size */
     Stream_Read_UINT16(input_stream, rdpsnd->server_timestamp);
@@ -215,25 +232,30 @@ void guac_rdpsnd_training_handler(guac_rdpsndPlugin* rdpsnd,
     Stream_Write_UINT16(output_stream, rdpsnd->server_timestamp);
     Stream_Write_UINT16(output_stream, data_size);
 
-    pthread_mutex_lock(&(guac_client_data->rdp_lock));
+    pthread_mutex_lock(&(client_data->rdp_lock));
     svc_plugin_send((rdpSvcPlugin*) rdpsnd, output_stream);
-    pthread_mutex_unlock(&(guac_client_data->rdp_lock));
+    pthread_mutex_unlock(&(client_data->rdp_lock));
 
 }
 
 void guac_rdpsnd_wave_info_handler(guac_rdpsndPlugin* rdpsnd,
-        guac_audio_stream* audio, wStream* input_stream,
-        guac_rdpsnd_pdu_header* header) {
+        wStream* input_stream, guac_rdpsnd_pdu_header* header) {
 
-    unsigned char buffer[4];
     int format;
 
+    /* Get associated client data */
+    guac_client* client = rdpsnd->client;
+    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
+
+    /* Get audio stream from client data */
+    guac_audio_stream* audio = client_data->audio;
+
     /* Read wave information */
     Stream_Read_UINT16(input_stream, rdpsnd->server_timestamp);
     Stream_Read_UINT16(input_stream, format);
     Stream_Read_UINT8(input_stream, rdpsnd->waveinfo_block_number);
     Stream_Seek(input_stream, 3);
-    Stream_Read(input_stream, buffer, 4);
+    Stream_Read(input_stream, rdpsnd->initial_wave_data, 4);
 
     /*
      * Size of incoming wave data is equal to the body size field of this
@@ -245,35 +267,42 @@ void guac_rdpsnd_wave_info_handler(guac_rdpsndPlugin* rdpsnd,
     /* Read wave in next iteration */
     rdpsnd->next_pdu_is_wave = TRUE;
 
-    /* Init stream with requested format */
-    guac_audio_stream_begin(audio,
-            rdpsnd->formats[format].rate,
-            rdpsnd->formats[format].channels,
-            rdpsnd->formats[format].bps);
-
-    /* Write initial 4 bytes of data */
-    guac_audio_stream_write_pcm(audio, buffer, 4);
+    /* Reset audio stream if format has changed */
+    if (audio != NULL)
+        guac_audio_stream_reset(audio, NULL,
+                rdpsnd->formats[format].rate,
+                rdpsnd->formats[format].channels,
+                rdpsnd->formats[format].bps);
 
 }
 
 void guac_rdpsnd_wave_handler(guac_rdpsndPlugin* rdpsnd,
-        guac_audio_stream* audio, wStream* input_stream,
-        guac_rdpsnd_pdu_header* header) {
+        wStream* input_stream, guac_rdpsnd_pdu_header* header) {
 
     rdpSvcPlugin* plugin = (rdpSvcPlugin*)rdpsnd;
 
-    rdp_guac_client_data* guac_client_data =
-        (rdp_guac_client_data*) audio->client->data;
+    /* Get associated client data */
+    guac_client* client = rdpsnd->client;
+    rdp_guac_client_data* client_data = (rdp_guac_client_data*) client->data;
+
+    /* Get audio stream from client data */
+    guac_audio_stream* audio = client_data->audio;
 
     /* Wave Confirmation PDU */
     wStream* output_stream = Stream_New(NULL, 8);
 
     /* Get wave data */
-    unsigned char* buffer = Stream_Buffer(input_stream) + 4;
+    unsigned char* buffer = Stream_Buffer(input_stream);
+
+    /* Copy over first four bytes */
+    memcpy(buffer, rdpsnd->initial_wave_data, 4);
 
     /* Write rest of audio packet */
-    guac_audio_stream_write_pcm(audio, buffer, rdpsnd->incoming_wave_size);
-    guac_audio_stream_end(audio);
+    if (audio != NULL) {
+        guac_audio_stream_write_pcm(audio, buffer,
+                rdpsnd->incoming_wave_size + 4);
+        guac_audio_stream_flush(audio);
+    }
 
     /* Write Wave Confirmation PDU */
     Stream_Write_UINT8(output_stream, SNDC_WAVECONFIRM);
@@ -284,9 +313,9 @@ void guac_rdpsnd_wave_handler(guac_rdpsndPlugin* rdpsnd,
     Stream_Write_UINT8(output_stream, 0);
 
     /* Send Wave Confirmation PDU */
-    pthread_mutex_lock(&(guac_client_data->rdp_lock));
+    pthread_mutex_lock(&(client_data->rdp_lock));
     svc_plugin_send(plugin, output_stream);
-    pthread_mutex_unlock(&(guac_client_data->rdp_lock));
+    pthread_mutex_unlock(&(client_data->rdp_lock));
 
     /* We no longer expect to receive wave data */
     rdpsnd->next_pdu_is_wave = FALSE;
@@ -294,10 +323,9 @@ void guac_rdpsnd_wave_handler(guac_rdpsndPlugin* rdpsnd,
 }
 
 void guac_rdpsnd_close_handler(guac_rdpsndPlugin* rdpsnd,
-        guac_audio_stream* audio, wStream* input_stream,
-        guac_rdpsnd_pdu_header* header) {
+        wStream* input_stream, guac_rdpsnd_pdu_header* header) {
 
-    /* STUB: Do nothing for now */
+    /* Do nothing */
 
 }
 
